@@ -20,7 +20,7 @@
 /*
  * Server packet routines to handle received packets.
  */
-/* $Id: packet_receive.c,v 1.135 2002/03/28 11:18:03 priikone Exp $ */
+/* $Id: packet_receive.c,v 1.135.2.1 2002/03/28 20:56:41 priikone Exp $ */
 
 #include "serverincludes.h"
 #include "server_internal.h"
@@ -73,7 +73,7 @@ void silc_server_notify(SilcServer server,
     if (dst_sock)
       /* Relay the packet */
       silc_server_relay_packet(server, dst_sock, idata->send_key,
-			       idata->hmac_receive, idata->psn_send++,
+			       idata->hmac_send, idata->psn_send++,
 			       packet, TRUE);
   }
 
