@@ -150,7 +150,7 @@ typedef struct {
   void *tmp;
 
   /* Reference count (when this reaches zero, config object is destroyed) */
-  SilcUInt16 refcount;
+  SilcInt16 refcount;
 
   /* The General section */
   char *module_path;
@@ -189,8 +189,7 @@ typedef struct {
 /* Prototypes */
 
 /* Basic config operations */
-SilcServerConfig silc_server_config_alloc(SilcServer server, 
-					  const char *filename);
+SilcServerConfig silc_server_config_alloc(const char *filename);
 void silc_server_config_destroy(SilcServerConfig config);
 void silc_server_config_ref(SilcServerConfigRef *ref, SilcServerConfig config,
 			    void *ref_ptr);
